/*
 This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2006.
 Authors: Andreas Knuepfer, Holger Brunst, Ronny Brendel, Thomas Kriebitzsch
*/

#ifndef OTF_FILEMANAGER_H
#define OTF_FILEMANAGER_H


#include <inttypes.h>


struct struct_OTF_File;
typedef struct struct_OTF_File OTF_File;


/** File Manager that schedules an unlimited number of OTF_Files to a limited
number of actual open OS files. Therefore all open files are registered with 
this manager. When a file is requested while no more OS files are available
any of the other files are suspended, i.e. the OS file is closed. 
Suspended files are silently re-opened on their next use. */
struct struct_OTF_FileManager;
typedef struct struct_OTF_FileManager OTF_FileManager;


/** Generates a new file manager with a maximum number of files that are allowed
to be open simultaneously. */
OTF_FileManager* OTF_FileManager_open( uint32_t number );

/** Closes the file manager */
void OTF_FileManager_close( OTF_FileManager* m );

/** Returns the number of files currently open. */
uint32_t OTF_FileManager_getCount( OTF_FileManager* m );

/** Returns the numbner of files allowed to be open simultaneously. */
uint32_t OTF_FileManager_getNumber( OTF_FileManager* m );

/** Sets the number of files allowed to be open simultaneously. */
uint32_t OTF_FileManager_setNumber( OTF_FileManager* m, uint32_t number );

/** Ensure there is a free file handle available after this call. 
return 1 on success, 0 otherwise (which is not supposed to happen) */
int OTF_FileManager_guaranteeFile( OTF_FileManager* m );

/** Registers the 'file' as open. Return 1 on success, 0 otherwise. */
int OTF_FileManager_registerFile( OTF_FileManager* m, OTF_File* file );

/** Marks currently opened 'file' as used which is important for the 
scheduling strategy, i.e. the internal decision which file to suspend next.
return 1 on success or 0 for an suspended file. */
int OTF_FileManager_touchFile( OTF_FileManager* m, OTF_File* file );

/** Suspend an open file explicitly. this may be called externaly or 
internally. Return 1 on success, 0 otherwise. */
int OTF_FileManager_suspendFile( OTF_FileManager* m, OTF_File* file );


#endif /* OTF_FILEMANAGER_H */
