/*
 This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2006.
 Authors: Andreas Knuepfer, Holger Brunst, Ronny Brendel, Thomas Kriebitzsch
*/

#include "config.h"

#include <cassert>

#include <otf.h>

#include "Handler.h"


/* *** Definition handler *** ************************************* */


int handleDefTimerresolution( void* firsthandlerarg, uint32_t streamid,
		uint64_t ticksPerSecond ) {


	Control* control= (Control*) firsthandlerarg;
	control->num++;

	if ( control->nodef ) 
		return OTF_RETURN_OK;

	if ( ( control->num < control->minNum ) || ( control->num > control->maxNum ) ) 
		return OTF_RETURN_OK;

	fprintf( stdout, "(#%llu) \tDefTimerResolution: stream %u, %llu ticksPerSecond\n", 
		(unsigned long long) control->num -1,
		streamid, (unsigned long long) ticksPerSecond );

	return OTF_RETURN_OK;
}


int handleDefProcess( void* firsthandlerarg, uint32_t streamid,
		uint32_t deftoken, const char* name, uint32_t parent ) {


	Control* control= (Control*) firsthandlerarg;
	control->num++;

	if ( control->nodef ) 
		return OTF_RETURN_OK;

	if ( ( control->num < control->minNum ) || ( control->num > control->maxNum ) ) 
		return OTF_RETURN_OK;

	fprintf( stdout, "(#%llu) \tDefProcess: stream %u, token %u \"%s\", parent %u\n",
		(unsigned long long) control->num -1,
		streamid, deftoken, name, parent );

	return OTF_RETURN_OK;
}


int handleDefFunction( void* firsthandlerarg, uint32_t streamid,
	uint32_t deftoken, const char* name, uint32_t group, uint32_t scltoken ) {


	Control* control= (Control*) firsthandlerarg;
	control->num++;

	if ( control->nodef ) 
		return OTF_RETURN_OK;

	if ( ( control->num < control->minNum ) || ( control->num > control->maxNum ) ) 
		return OTF_RETURN_OK;

	fprintf( stdout, "(#%llu) \tDefFunction: stream %u, token %u \"%s\", group %u, scl %u\n", 
		(unsigned long long) control->num -1,
		streamid, deftoken, name, group, scltoken );

	return OTF_RETURN_OK;
}


int handleDefinitionComment( void* firsthandlerarg, uint32_t stream, const char* comment ) {


	Control* control= (Control*) firsthandlerarg;
	control->num++;

	if ( control->nodef ) 
		return OTF_RETURN_OK;

	if ( ( control->num < control->minNum ) || ( control->num > control->maxNum ) ) 
		return OTF_RETURN_OK;

	fprintf( stdout, "(#%llu) \tDefComment: stream %u, \"%s\"\n", 
		(unsigned long long) control->num -1, stream, comment );

	return OTF_RETURN_OK;
}


int handleDefProcessGroup( void* firsthandlerarg, uint32_t stream, uint32_t procGroup, 
		const char* name, uint32_t numberOfProcs, const uint32_t* procs ) {


	Control* control= (Control*) firsthandlerarg;
	control->num++;

	if ( control->nodef ) 
		return OTF_RETURN_OK;

	if ( ( control->num < control->minNum ) || ( control->num > control->maxNum ) ) 
		return OTF_RETURN_OK;

	fprintf( stdout, "(#%llu) \tDefProcessGroup: stream %u, token %u \"%s\", num %u:", 
		(unsigned long long) control->num -1,
		stream, procGroup, name, numberOfProcs );

	for ( uint32_t i= 0; i < numberOfProcs; i++ ) {
	
		fprintf( stdout, " %u", procs[i] );
	}
	fprintf( stdout, "\n" );

	return OTF_RETURN_OK;
}


int handleDefFunctionGroup( void* firsthandlerarg, uint32_t stream, 
		uint32_t funcGroup, const char* name ) {


	Control* control= (Control*) firsthandlerarg;
	control->num++;

	if ( control->nodef ) 
		return OTF_RETURN_OK;

	if ( ( control->num < control->minNum ) || ( control->num > control->maxNum ) ) 
		return OTF_RETURN_OK;

	fprintf( stdout, "(#%llu) \tDefFunctionGroup: stream %u, token %u \"%s\"\n", 
		(unsigned long long) control->num -1,
		stream, funcGroup, name );

	return OTF_RETURN_OK;
}


int handleDefCollectiveOperation( void* firsthandlerarg, uint32_t stream,
		uint32_t collOp, const char* name, uint32_t type ) {


	Control* control= (Control*) firsthandlerarg;
	control->num++;

	if ( control->nodef ) 
		return OTF_RETURN_OK;

	if ( ( control->num < control->minNum ) || ( control->num > control->maxNum ) ) 
		return OTF_RETURN_OK;

	fprintf( stdout, "(#%llu) \tDefCollectiveOperation: stream %u, token %u \"%s\", type %u\n", 
		(unsigned long long) control->num -1,
		stream, collOp, name, type );

	return OTF_RETURN_OK;
}


int handleDefCounter( void* firsthandlerarg, uint32_t stream,
		uint32_t counter, const char* name, uint32_t properties,
		uint32_t counterGroup, const char* unit ) {


	Control* control= (Control*) firsthandlerarg;
	control->num++;

	if ( control->nodef ) 
		return OTF_RETURN_OK;

	if ( ( control->num < control->minNum ) || ( control->num > control->maxNum ) ) 
		return OTF_RETURN_OK;

	fprintf( stdout, "(#%llu) \tDefCounter: stream %u, token %u \"%s\", "
		"properties %u, group %u, unit \"%s\"\n", 
		(unsigned long long) control->num -1,
		stream, counter, name, properties, counterGroup, unit );

	return OTF_RETURN_OK;
}


int handleDefCounterGroup( void* firsthandlerarg, uint32_t stream,
		uint32_t counterGroup, const char* name ) {


	Control* control= (Control*) firsthandlerarg;
	control->num++;

	if ( control->nodef ) 
		return OTF_RETURN_OK;

	if ( ( control->num < control->minNum ) || ( control->num > control->maxNum ) ) 
		return OTF_RETURN_OK;

	fprintf( stdout, "(#%llu) \tDefCounterGroup: stream %u, token %u \"%s\"\n", 
		(unsigned long long) control->num -1,
		stream, counterGroup, name );

	return OTF_RETURN_OK;
}


int handleDefScl( void* firsthandlerarg, uint32_t stream,
		uint32_t source, uint32_t sourceFile, uint32_t line ) {


	Control* control= (Control*) firsthandlerarg;
	control->num++;

	if ( control->nodef ) 
		return OTF_RETURN_OK;

	if ( ( control->num < control->minNum ) || ( control->num > control->maxNum ) ) 
		return OTF_RETURN_OK;

	fprintf( stdout, "(#%llu) \tDefScl: stream %u, token %u, line %u, file %u\n", 
		(unsigned long long) control->num -1,
		stream, source, line, sourceFile );

	return OTF_RETURN_OK;
}


int handleDefSclFile( void* firsthandlerarg, uint32_t stream,
		uint32_t sourceFile, const char* name ) {


	Control* control= (Control*) firsthandlerarg;
	control->num++;

	if ( control->nodef ) 
		return OTF_RETURN_OK;

	if ( ( control->num < control->minNum ) || ( control->num > control->maxNum ) ) 
		return OTF_RETURN_OK;

	fprintf( stdout, "(#%llu) \tDefSclFile: stream %u, token %u \"%s\"\n", 
		(unsigned long long) control->num -1,
		stream, sourceFile, name );

	return OTF_RETURN_OK;
}


int handleDefCreator( void* firsthandlerarg, uint32_t stream, const char* creator ) {


	Control* control= (Control*) firsthandlerarg;
	control->num++;

	if ( control->nodef ) 
		return OTF_RETURN_OK;

	if ( ( control->num < control->minNum ) || ( control->num > control->maxNum ) ) 
		return OTF_RETURN_OK;

	fprintf( stdout, "(#%llu) \tDefCreator: stream %u, \"%s\"\n", 
		(unsigned long long) control->num -1,
		stream, creator );

	return OTF_RETURN_OK;
}



/* *** Event handler *** ****************************************** */


int handleCounter( void* firsthandlerarg, uint64_t time, uint32_t process,
		uint32_t token, uint64_t value ) {


	Control* control= (Control*) firsthandlerarg;
	control->num++;

	if ( ( control->num < control->minNum ) || ( control->num > control->maxNum ) ) 
		return OTF_RETURN_OK;

	if ( ( time < control->minTime ) || ( time > control->maxTime ) ) 
		return OTF_RETURN_OK;

	fprintf( stdout, "(#%llu) \t%llu Counter: process %u, token %u, value %llu\n", 
		(unsigned long long) control->num -1,
		(unsigned long long) time, process,	token, (unsigned long long) value );

	return OTF_RETURN_OK;
}


int handleEnter( void* firsthandlerarg, uint64_t time, uint32_t statetoken,
		uint32_t cpuid, uint32_t scltoken ) {


	Control* control= (Control*) firsthandlerarg;
	control->num++;

	if ( ( control->num < control->minNum ) || ( control->num > control->maxNum ) ) 
		return OTF_RETURN_OK;

	if ( ( time < control->minTime ) || ( time > control->maxTime ) ) 
		return OTF_RETURN_OK;

	fprintf( stdout, "(#%llu) \t%llu Enter: process %u, state %u, scl %u\n", 
		(unsigned long long) control->num -1,
		(unsigned long long) time, cpuid, statetoken, scltoken );

	return OTF_RETURN_OK;
}


int handleLeave( void* firsthandlerarg, uint64_t time, uint32_t statetoken,
		uint32_t cpuid, uint32_t scltoken ) {


	Control* control= (Control*) firsthandlerarg;
	control->num++;

	if ( ( control->num < control->minNum ) || ( control->num > control->maxNum ) ) 
		return OTF_RETURN_OK;

	if ( ( time < control->minTime ) || ( time > control->maxTime ) ) 
		return OTF_RETURN_OK;

	fprintf( stdout, "(#%llu) \t%llu Leave: process %u, state %u, scl %u\n", 
		(unsigned long long) control->num -1,
		(unsigned long long) time, cpuid, statetoken, scltoken );

	return OTF_RETURN_OK;
}


int handleRecvmsg( void* firsthandlerarg, uint64_t time, uint32_t receiver,
		uint32_t sender, uint32_t procGroup, uint32_t msgtag, 
		uint32_t msglength, uint32_t scltoken ) {


	Control* control= (Control*) firsthandlerarg;
	control->num++;

	if ( ( control->num < control->minNum ) || ( control->num > control->maxNum ) ) 
		return OTF_RETURN_OK;

	if ( ( time < control->minTime ) || ( time > control->maxTime ) ) 
		return OTF_RETURN_OK;

	fprintf( stdout, "(#%llu) \t%llu Receive: sender %u, receiver %u, communicator %u, tag %u, length %u, scl %u\n", 
		(unsigned long long) control->num -1,
		(unsigned long long) time, sender, receiver, procGroup, msgtag, msglength, scltoken );

	return OTF_RETURN_OK;
}


int handleSendmsg( void* firsthandlerarg, uint64_t time, uint32_t sender,
		uint32_t receiver, uint32_t procGroup, uint32_t msgtag, 
		uint32_t msglength, uint32_t scltoken ) {


	Control* control= (Control*) firsthandlerarg;
	control->num++;

	if ( ( control->num < control->minNum ) || ( control->num > control->maxNum ) ) 
		return OTF_RETURN_OK;

	if ( ( time < control->minTime ) || ( time > control->maxTime ) ) 
		return OTF_RETURN_OK;

	fprintf( stdout, "(#%llu) \t%llu Send: sender %u, receiver %u, communicator %u, tag %u, length %u, scl %u\n", 
		(unsigned long long) control->num -1,
		(unsigned long long) time, sender, receiver, procGroup, msgtag, msglength, scltoken );

	return OTF_RETURN_OK;
}


int handleBeginProcess( void* firsthandlerarg, uint64_t time, uint32_t process ) {


	Control* control= (Control*) firsthandlerarg;
	control->num++;

	if ( ( control->num < control->minNum ) || ( control->num > control->maxNum ) ) 
		return OTF_RETURN_OK;

	if ( ( time < control->minTime ) || ( time > control->maxTime ) ) 
		return OTF_RETURN_OK;

	fprintf( stdout, "(#%llu) \t%llu BeginProcess: process %u\n", 
		(unsigned long long) control->num -1,
		(unsigned long long) time, process );

	return OTF_RETURN_OK;
}


int handleEndProcess( void* firsthandlerarg, uint64_t time, uint32_t process ) {


	Control* control= (Control*) firsthandlerarg;
	control->num++;

	if ( ( control->num < control->minNum ) || ( control->num > control->maxNum ) ) 
		return OTF_RETURN_OK;

	if ( ( time < control->minTime ) || ( time > control->maxTime ) ) 
		return OTF_RETURN_OK;

	fprintf( stdout, "(#%llu) \t%llu EndProcess: process %u\n", 
		(unsigned long long) control->num -1,
		(unsigned long long) time, process );

	return OTF_RETURN_OK;
}


int handleEventComment( void* firsthandlerarg, uint64_t time, 
		const char* comment ) {


	Control* control= (Control*) firsthandlerarg;
	control->num++;

	if ( ( control->num < control->minNum ) || ( control->num > control->maxNum ) ) 
		return OTF_RETURN_OK;

	if ( ( time < control->minTime ) || ( time > control->maxTime ) ) 
		return OTF_RETURN_OK;

	fprintf( stdout, "(#%llu) \t%llu Comment: \"%s\"\n", 
		(unsigned long long) control->num -1,
		(unsigned long long) time, comment );

	return OTF_RETURN_OK;
}


int handleCollectiveOperation( void* firsthandlerarg, uint64_t time, 
    uint32_t process, uint32_t functionToken, uint32_t communicator, 
    uint32_t rootprocess, uint32_t sent, uint32_t received, 
    uint64_t duration, uint32_t scltoken ) {


	Control* control= (Control*) firsthandlerarg;
	control->num++;

	if ( ( control->num < control->minNum ) || ( control->num > control->maxNum ) ) 
		return OTF_RETURN_OK;

	if ( ( time < control->minTime ) || ( time > control->maxTime ) ) 
		return OTF_RETURN_OK;

	fprintf( stdout, "(#%llu) \t%llu Collective:process %u, token %u, communicator %u, root %u, "
		"%u bytes send, %u bytes received, duration %llu, scl %u\n", 
		(unsigned long long) control->num -1,
		(unsigned long long) time, process, functionToken, communicator, 
		rootprocess, sent, received, (unsigned long long) duration, scltoken);

	return OTF_RETURN_OK;
}

